@@ -1,6 +1,6 @@
 /*
 
-   BLIS    
+   BLIS
    An object-based framework for developing high-performance BLAS-like
    libraries.
 
@@ -90,41 +90,51 @@ static func_t packm_struc_cxk_kers[BLIS_NUM_PACK_SCHEMA_TYPES] =
 // 0111 row/col panels: real+imaginary only
                { { NULL,                      bli_cpackm_struc_cxk_rih,
                    NULL,                      bli_zpackm_struc_cxk_rih,  } },
+// 1000 row/col panels: 1m-expanded (1e)
+               { { NULL,                      bli_cpackm_struc_cxk_1er,
+                   NULL,                      bli_zpackm_struc_cxk_1er,  } },
+// 1001 row/col panels: 1m-reordered (1r)
+               { { NULL,                      bli_cpackm_struc_cxk_1er,
+                   NULL,                      bli_zpackm_struc_cxk_1er,  } },
 };
 
 
-void bli_packm_blk_var1( obj_t*   c,
-                         obj_t*   p,
-                         cntx_t*  cntx,
-                         thrinfo_t* t )
+void bli_packm_blk_var1
+     (
+       obj_t*   c,
+       obj_t*   p,
+       cntx_t*  cntx,
+       cntl_t*  cntl,
+       thrinfo_t* t
+     )
 {
-	num_t     dt_cp      = bli_obj_datatype( *c );
+	num_t     dt_cp      = bli_obj_dt( c );
 
-	struc_t   strucc     = bli_obj_struc( *c );
-	doff_t    diagoffc   = bli_obj_diag_offset( *c );
-	diag_t    diagc      = bli_obj_diag( *c );
-	uplo_t    uploc      = bli_obj_uplo( *c );
-	trans_t   transc     = bli_obj_conjtrans_status( *c );
-	pack_t    schema     = bli_obj_pack_schema( *p );
-	bool_t    invdiag    = bli_obj_has_inverted_diag( *p );
-	bool_t    revifup    = bli_obj_is_pack_rev_if_upper( *p );
-	bool_t    reviflo    = bli_obj_is_pack_rev_if_lower( *p );
+	struc_t   strucc     = bli_obj_struc( c );
+	doff_t    diagoffc   = bli_obj_diag_offset( c );
+	diag_t    diagc      = bli_obj_diag( c );
+	uplo_t    uploc      = bli_obj_uplo( c );
+	trans_t   transc     = bli_obj_conjtrans_status( c );
+	pack_t    schema     = bli_obj_pack_schema( p );
+	bool_t    invdiag    = bli_obj_has_inverted_diag( p );
+	bool_t    revifup    = bli_obj_is_pack_rev_if_upper( p );
+	bool_t    reviflo    = bli_obj_is_pack_rev_if_lower( p );
 
-	dim_t     m_p        = bli_obj_length( *p );
-	dim_t     n_p        = bli_obj_width( *p );
-	dim_t     m_max_p    = bli_obj_padded_length( *p );
-	dim_t     n_max_p    = bli_obj_padded_width( *p );
+	dim_t     m_p        = bli_obj_length( p );
+	dim_t     n_p        = bli_obj_width( p );
+	dim_t     m_max_p    = bli_obj_padded_length( p );
+	dim_t     n_max_p    = bli_obj_padded_width( p );
 
-	void*     buf_c      = bli_obj_buffer_at_off( *c );
-	inc_t     rs_c       = bli_obj_row_stride( *c );
-	inc_t     cs_c       = bli_obj_col_stride( *c );
+	void*     buf_c      = bli_obj_buffer_at_off( c );
+	inc_t     rs_c       = bli_obj_row_stride( c );
+	inc_t     cs_c       = bli_obj_col_stride( c );
 
-	void*     buf_p      = bli_obj_buffer_at_off( *p );
-	inc_t     rs_p       = bli_obj_row_stride( *p );
-	inc_t     cs_p       = bli_obj_col_stride( *p );
-	inc_t     is_p       = bli_obj_imag_stride( *p );
-	dim_t     pd_p       = bli_obj_panel_dim( *p );
-	inc_t     ps_p       = bli_obj_panel_stride( *p );
+	void*     buf_p      = bli_obj_buffer_at_off( p );
+	inc_t     rs_p       = bli_obj_row_stride( p );
+	inc_t     cs_p       = bli_obj_col_stride( p );
+	inc_t     is_p       = bli_obj_imag_stride( p );
+	dim_t     pd_p       = bli_obj_panel_dim( p );
+	inc_t     ps_p       = bli_obj_panel_stride( p );
 
 	obj_t     kappa;
 	obj_t*    kappa_p;
@@ -140,12 +150,12 @@ void bli_packm_blk_var1( obj_t*   c,
 	// whether we are executing an induced method.
 	if ( bli_is_nat_packed( schema ) )
 	{
-		// This branch if for native execution, where we assume that
+		// This branch is for native execution, where we assume that
 		// the micro-kernel will always apply the alpha scalar of the
 		// higher-level operation. Thus, we use BLIS_ONE for kappa so
 		// that the underlying packm implementation does not perform
 		// any scaling during packing.
-		buf_kappa = bli_obj_buffer_for_const( dt_cp, BLIS_ONE );
+		buf_kappa = bli_obj_buffer_for_const( dt_cp, &BLIS_ONE );
 	}
 	else // if ( bli_is_ind_packed( schema ) )
 	{
@@ -156,31 +166,28 @@ void bli_packm_blk_var1( obj_t*   c,
 		// real domain micro-kernels. (In the aforementioned situation,
 		// applying a real scalar is easy, but applying a complex one is
 		// harder, so we avoid the need altogether with the code below.)
-		if( bli_thread_am_ochief( t ) )
+		if ( bli_obj_scalar_has_nonzero_imag( p ) )
 		{
-			if ( bli_obj_scalar_has_nonzero_imag( p ) )
-			{
-//printf( "applying non-zero imag kappa\n" );
-				// Detach the scalar.
-				bli_obj_scalar_detach( p, &kappa );
-	
-				// Reset the attached scalar (to 1.0).
-				bli_obj_scalar_reset( p );
-	
-				kappa_p = &kappa;
-			}
-			else
-			{
-				// If the internal scalar of A has only a real component, then
-				// we will apply it later (in the micro-kernel), and so we will
-				// use BLIS_ONE to indicate no scaling during packing.
-				kappa_p = &BLIS_ONE;
-			}
+			//printf( "applying non-zero imag kappa\n" );
+
+			// Detach the scalar.
+			bli_obj_scalar_detach( p, &kappa );
+
+			// Reset the attached scalar (to 1.0).
+			bli_obj_scalar_reset( p );
+
+			kappa_p = &kappa;
+		}
+		else
+		{
+			// If the internal scalar of A has only a real component, then
+			// we will apply it later (in the micro-kernel), and so we will
+			// use BLIS_ONE to indicate no scaling during packing.
+			kappa_p = &BLIS_ONE;
 		}
-		kappa_p = bli_thread_obroadcast( t, kappa_p );
 	
 		// Acquire the buffer to the kappa chosen above.
-		buf_kappa = bli_obj_buffer_for_1x1( dt_cp, *kappa_p );
+		buf_kappa = bli_obj_buffer_for_1x1( dt_cp, kappa_p );
 	}
 
 
@@ -194,7 +201,12 @@ void bli_packm_blk_var1( obj_t*   c,
 	         bli_is_rpi_packed( schema ) )  packm_kers = packm_struc_cxk_rih_kers;
 	else                                    packm_kers = packm_struc_cxk_kers;
 #else
-	func_t* cntx_packm_kers = bli_cntx_get_packm_ukr( cntx );
+	// The original idea here was to read the packm_ukr from the context
+	// if it is non-NULL. The problem is, it requires that we be able to
+	// assume that the packm_ukr field is initialized to NULL, which it
+	// currently is not.
+
+	//func_t* cntx_packm_kers = bli_cntx_get_packm_ukr( cntx );
 
 	//if ( bli_func_is_null_dt( dt_cp, cntx_packm_kers ) )
 	{
@@ -203,7 +215,6 @@ void bli_packm_blk_var1( obj_t*   c,
 		// we use the default lookup table to determine the right func_t
 		// for the current schema.
 		const dim_t i = bli_pack_schema_index( schema );
-//printf( "bli_packm_blk_var1: pack schema index = %lu (schema = %x)\n", i, schema );
 
 		packm_kers = &packm_struc_cxk_kers[ i ];
 	}
@@ -221,11 +232,6 @@ void bli_packm_blk_var1( obj_t*   c,
 	// Query the datatype-specific function pointer from the func_t object.
 	packm_ker = bli_func_get_dt( dt_cp, packm_kers );
 
-
-//bli_cntx_print( cntx );
-//printf( "bli_packm_blk_var1: packm_ker = %p\n", packm_ker );
-//printf( "bli_packm_blk_var1: cntx_packm_ker = %p\n", cntx_packm_kers );
-//printf( "bli_packm_blk_var1: local_table_entry = %p\n", &packm_struc_cxk_kers[ bli_pack_schema_index( schema ) ] );
 	// Index into the type combination array to extract the correct
 	// function pointer.
 	f = ftypes[dt_cp];
@@ -283,7 +289,7 @@ void PASTEMAC(ch,varname) \
        thrinfo_t* thread  \
      ) \
 { \
-	PASTECH2(ch,opname,_ft) packm_ker_cast = packm_ker; \
+	PASTECH2(ch,opname,_ker_ft) packm_ker_cast = packm_ker; \
 \
 	ctype* restrict kappa_cast = kappa; \
 	ctype* restrict c_cast     = c; \
@@ -338,10 +344,10 @@ void PASTEMAC(ch,varname) \
 	   express the remaining parameters and code. */ \
 	if ( bli_does_trans( transc ) ) \
 	{ \
-		bli_swap_incs( rs_c, cs_c ); \
-		bli_negate_diag_offset( diagoffc ); \
-		bli_toggle_uplo( uploc ); \
-		bli_toggle_trans( transc ); \
+		bli_swap_incs( &rs_c, &cs_c ); \
+		bli_negate_diag_offset( &diagoffc ); \
+		bli_toggle_uplo( &uploc ); \
+		bli_toggle_trans( &transc ); \
 	} \
 \
 	/* Create flags to incidate row or column storage. Note that the
@@ -598,6 +604,57 @@ PASTEMAC(ch,fprintm)( stdout, "packm_var2: a", m, n, \
 			p_inc = ps_p; \
 		} \
 \
+/*
+if ( col_stored ) { \
+	if ( bli_thread_work_id( thread ) == 0 ) \
+	{ \
+	printf( "packm_blk_var1: thread %lu  (a = %p, ap = %p)\n", bli_thread_work_id( thread ), c_use, p_use ); \
+	fflush( stdout ); \
+	PASTEMAC(ch,fprintm)( stdout, "packm_blk_var1: a", *m_panel_use, *n_panel_use, \
+	                      ( ctype* )c_use,         rs_c, cs_c, "%4.1f", "" ); \
+	PASTEMAC(ch,fprintm)( stdout, "packm_blk_var1: ap", *m_panel_max, *n_panel_max, \
+	                      ( ctype* )p_use,         rs_p, cs_p, "%4.1f", "" ); \
+	fflush( stdout ); \
+	} \
+bli_thread_obarrier( thread ); \
+	if ( bli_thread_work_id( thread ) == 1 ) \
+	{ \
+	printf( "packm_blk_var1: thread %lu  (a = %p, ap = %p)\n", bli_thread_work_id( thread ), c_use, p_use ); \
+	fflush( stdout ); \
+	PASTEMAC(ch,fprintm)( stdout, "packm_blk_var1: a", *m_panel_use, *n_panel_use, \
+	                      ( ctype* )c_use,         rs_c, cs_c, "%4.1f", "" ); \
+	PASTEMAC(ch,fprintm)( stdout, "packm_blk_var1: ap", *m_panel_max, *n_panel_max, \
+	                      ( ctype* )p_use,         rs_p, cs_p, "%4.1f", "" ); \
+	fflush( stdout ); \
+	} \
+bli_thread_obarrier( thread ); \
+} \
+else { \
+	if ( bli_thread_work_id( thread ) == 0 ) \
+	{ \
+	printf( "packm_blk_var1: thread %lu  (b = %p, bp = %p)\n", bli_thread_work_id( thread ), c_use, p_use ); \
+	fflush( stdout ); \
+	PASTEMAC(ch,fprintm)( stdout, "packm_blk_var1: b", *m_panel_use, *n_panel_use, \
+	                      ( ctype* )c_use,         rs_c, cs_c, "%4.1f", "" ); \
+	PASTEMAC(ch,fprintm)( stdout, "packm_blk_var1: bp", *m_panel_max, *n_panel_max, \
+	                      ( ctype* )p_use,         rs_p, cs_p, "%4.1f", "" ); \
+	fflush( stdout ); \
+	} \
+bli_thread_obarrier( thread ); \
+	if ( bli_thread_work_id( thread ) == 1 ) \
+	{ \
+	printf( "packm_blk_var1: thread %lu  (b = %p, bp = %p)\n", bli_thread_work_id( thread ), c_use, p_use ); \
+	fflush( stdout ); \
+	PASTEMAC(ch,fprintm)( stdout, "packm_blk_var1: b", *m_panel_use, *n_panel_use, \
+	                      ( ctype* )c_use,         rs_c, cs_c, "%4.1f", "" ); \
+	PASTEMAC(ch,fprintm)( stdout, "packm_blk_var1: bp", *m_panel_max, *n_panel_max, \
+	                      ( ctype* )p_use,         rs_p, cs_p, "%4.1f", "" ); \
+	fflush( stdout ); \
+	} \
+bli_thread_obarrier( thread ); \
+} \
+*/ \
+\
 /*
 		if ( bli_is_4mi_packed( schema ) ) { \
 		printf( "packm_var2: is_p_use = %lu\n", is_p_use ); \
