@@ -1,6 +1,6 @@
 /*
 
-   BLIS    
+   BLIS
    An object-based framework for developing high-performance BLAS-like
    libraries.
 
@@ -36,56 +36,62 @@
 
 #define FUNCPTR_T gemm_fp
 
-typedef void (*FUNCPTR_T)(
-                           doff_t  diagoffb,
-                           pack_t  schema_a,
-                           pack_t  schema_b,
-                           dim_t   m,
-                           dim_t   n,
-                           dim_t   k,
-                           void*   alpha1,
-                           void*   a, inc_t cs_a, dim_t pd_a, inc_t ps_a,
-                           void*   b, inc_t rs_b, dim_t pd_b, inc_t ps_b,
-                           void*   alpha2,
-                           void*   c, inc_t rs_c, inc_t cs_c,
-                           cntx_t* cntx,
-                           thrinfo_t* thread
-                         );
+typedef void (*FUNCPTR_T)
+     (
+       doff_t  diagoffb,
+       pack_t  schema_a,
+       pack_t  schema_b,
+       dim_t   m,
+       dim_t   n,
+       dim_t   k,
+       void*   alpha1,
+       void*   a, inc_t cs_a, dim_t pd_a, inc_t ps_a,
+       void*   b, inc_t rs_b, dim_t pd_b, inc_t ps_b,
+       void*   alpha2,
+       void*   c, inc_t rs_c, inc_t cs_c,
+       cntx_t* cntx,
+       rntm_t* rntm,
+       thrinfo_t* thread
+     );
 
 static FUNCPTR_T GENARRAY(ftypes,trsm_rl_ker_var2);
 
 
-void bli_trsm_rl_ker_var2( obj_t*  a,
-                           obj_t*  b,
-                           obj_t*  c,
-                           cntx_t* cntx,
-                           trsm_t* cntl,
-                           thrinfo_t* thread )
+void bli_trsm_rl_ker_var2
+     (
+       obj_t*  a,
+       obj_t*  b,
+       obj_t*  c,
+       cntx_t* cntx,
+       rntm_t* rntm,
+       cntl_t* cntl,
+       thrinfo_t* thread
+     )
 {
-	num_t     dt_exec   = bli_obj_execution_datatype( *c );
+	num_t     dt_exec   = bli_obj_exec_dt( c );
 
-	doff_t    diagoffb  = bli_obj_diag_offset( *b );
+	doff_t    diagoffb  = bli_obj_diag_offset( b );
 
-	pack_t    schema_a  = bli_obj_pack_schema( *a );
-	pack_t    schema_b  = bli_obj_pack_schema( *b );
+	pack_t    schema_a  = bli_obj_pack_schema( a );
+	pack_t    schema_b  = bli_obj_pack_schema( b );
 
-	dim_t     m         = bli_obj_length( *c );
-	dim_t     n         = bli_obj_width( *c );
-	dim_t     k         = bli_obj_width( *a );
+	dim_t     m         = bli_obj_length( c );
+	dim_t     n         = bli_obj_width( c );
+	dim_t     k         = bli_obj_width( a );
 
-	void*     buf_a     = bli_obj_buffer_at_off( *a );
-	inc_t     cs_a      = bli_obj_col_stride( *a );
-	dim_t     pd_a      = bli_obj_panel_dim( *a );
-	inc_t     ps_a      = bli_obj_panel_stride( *a );
+	void*     buf_a     = bli_obj_buffer_at_off( a );
+	inc_t     cs_a      = bli_obj_col_stride( a );
+	dim_t     pd_a      = bli_obj_panel_dim( a );
+	inc_t     ps_a      = bli_obj_panel_stride( a );
 
-	void*     buf_b     = bli_obj_buffer_at_off( *b );
-	inc_t     rs_b      = bli_obj_row_stride( *b );
-	dim_t     pd_b      = bli_obj_panel_dim( *b );
-	inc_t     ps_b      = bli_obj_panel_stride( *b );
+	void*     buf_b     = bli_obj_buffer_at_off( b );
+	inc_t     rs_b      = bli_obj_row_stride( b );
+	dim_t     pd_b      = bli_obj_panel_dim( b );
+	inc_t     ps_b      = bli_obj_panel_stride( b );
 
-	void*     buf_c     = bli_obj_buffer_at_off( *c );
-	inc_t     rs_c      = bli_obj_row_stride( *c );
-	inc_t     cs_c      = bli_obj_col_stride( *c );
+	void*     buf_c     = bli_obj_buffer_at_off( c );
+	inc_t     rs_c      = bli_obj_row_stride( c );
+	inc_t     cs_c      = bli_obj_col_stride( c );
 
 	void*     buf_alpha1;
 	void*     buf_alpha2;
@@ -93,11 +99,12 @@ void bli_trsm_rl_ker_var2( obj_t*  a,
 	FUNCPTR_T f;
 
 	// Grab the address of the internal scalar buffer for the scalar
-	// attached to A. This will be the alpha scalar used in the gemmtrsm
-	// subproblems (ie: the scalar that would be applied to the packed
-	// copy of A prior to it being updated by the trsm subproblem). This
-	// scalar may be unit, if for example it was applied during packing.
-	buf_alpha1 = bli_obj_internal_scalar_buffer( *a );
+	// attached to A (the non-triangular matrix). This will be the alpha
+	// scalar used in the gemmtrsm subproblems (ie: the scalar that would
+	// be applied to the packed copy of A prior to it being updated by
+	// the trsm subproblem). This scalar may be unit, if for example it
+	// was applied during packing.
+	buf_alpha1 = bli_obj_internal_scalar_buffer( a );
 
 	// Grab the address of the internal scalar buffer for the scalar
 	// attached to C. This will be the "beta" scalar used in the gemm-only
@@ -105,7 +112,7 @@ void bli_trsm_rl_ker_var2( obj_t*  a,
 	// the diagonal. We need this separate scalar because it's possible
 	// that the alpha attached to B was reset, if it was applied during
 	// packing.
-	buf_alpha2 = bli_obj_internal_scalar_buffer( *c );
+	buf_alpha2 = bli_obj_internal_scalar_buffer( c );
 
 	// Index into the type combination array to extract the correct
 	// function pointer.
@@ -124,6 +131,7 @@ void bli_trsm_rl_ker_var2( obj_t*  a,
 	   buf_alpha2,
 	   buf_c, rs_c, cs_c,
 	   cntx,
+	   rntm,
 	   thread );
 }
 
@@ -145,6 +153,7 @@ void PASTEMAC(ch,varname) \
        void*   alpha2, \
        void*   c, inc_t rs_c, inc_t cs_c, \
        cntx_t* cntx, \
+       rntm_t* rntm, \
        thrinfo_t* thread  \
      ) \
 { \
@@ -163,16 +172,20 @@ void PASTEMAC(ch,varname) \
 	   is transposed so that all kernel instances are of the "left"
 	   variety (since those are the only trsm ukernels that exist). */ \
 	PASTECH(ch,gemmtrsm_ukr_ft) \
-	               gemmtrsm_ukr = bli_cntx_get_l3_ukr_dt( dt, BLIS_GEMMTRSM_U_UKR, cntx ); \
+	               gemmtrsm_ukr = bli_cntx_get_l3_vir_ukr_dt( dt, BLIS_GEMMTRSM_U_UKR, cntx ); \
 	PASTECH(ch,gemm_ukr_ft) \
-	                   gemm_ukr = bli_cntx_get_l3_ukr_dt( dt, BLIS_GEMM_UKR, cntx ); \
+	                   gemm_ukr = bli_cntx_get_l3_vir_ukr_dt( dt, BLIS_GEMM_UKR, cntx ); \
 \
-	/* Temporary C buffer for edge cases. */ \
+	/* Temporary C buffer for edge cases. Note that the strides of this
+	   temporary buffer are set so that they match the storage of the
+	   original C matrix. For example, if C is column-stored, ct will be
+	   column-stored as well. */ \
 	ctype           ct[ BLIS_STACK_BUF_MAX_SIZE \
 	                    / sizeof( ctype ) ] \
 	                    __attribute__((aligned(BLIS_STACK_BUF_ALIGN_SIZE))); \
-	const inc_t     rs_ct       = 1; \
-	const inc_t     cs_ct       = MR; \
+	const bool_t    col_pref    = bli_cntx_l3_vir_ukr_prefers_cols_dt( dt, BLIS_GEMM_UKR, cntx ); \
+	const inc_t     rs_ct       = ( col_pref ? 1 : NR ); \
+	const inc_t     cs_ct       = ( col_pref ? MR : 1 ); \
 \
 	ctype* restrict zero        = PASTEMAC(ch,0); \
 	ctype* restrict minus_one   = PASTEMAC(ch,m1); \
@@ -348,13 +361,13 @@ void PASTEMAC(ch,varname) \
 	/* Save the pack schemas of A and B to the auxinfo_t object.
 	   NOTE: We swap the values for A and B since the triangular
 	   "A" matrix is actually contained within B. */ \
-	bli_auxinfo_set_schema_a( schema_b, aux ); \
-	bli_auxinfo_set_schema_b( schema_a, aux ); \
+	bli_auxinfo_set_schema_a( schema_b, &aux ); \
+	bli_auxinfo_set_schema_b( schema_a, &aux ); \
 \
 	/* Save the imaginary stride of A to the auxinfo_t object.
 	   NOTE: We swap the values for A and B since the triangular
 	   "A" matrix is actually contained within B. */ \
-	bli_auxinfo_set_is_b( istep_a, aux ); \
+	bli_auxinfo_set_is_b( istep_a, &aux ); \
 \
 	b1 = b_cast; \
 	c1 = c_cast; \
@@ -398,7 +411,7 @@ void PASTEMAC(ch,varname) \
 			   panel B21. */ \
 			b11       = b1; \
 			/* b21 = b1 + ( k_b11 * PACKNR ) / off_scl; */ \
-			bli_ptr_add( b21, b1, k_b11 * PACKNR, off_scl ); \
+			b21 = bli_ptr_inc_by_frac( b1, sizeof( ctype ), k_b11 * PACKNR, off_scl ); \
 \
 			/* Compute the panel stride for the current micro-panel. */ \
 			is_b_cur  = k_b1121 * PACKNR; \
@@ -409,12 +422,12 @@ void PASTEMAC(ch,varname) \
 			   object.
 			   NOTE: We swap the values for A and B since the triangular
 			   "A" matrix is actually contained within B. */ \
-			bli_auxinfo_set_is_a( is_b_cur, aux ); \
+			bli_auxinfo_set_is_a( is_b_cur, &aux ); \
 \
 			/* Loop over the m dimension (MR rows at a time). */ \
 			for ( i = 0; i < m_iter; ++i ) \
 			{ \
-				if( trsm_my_iter( i, thread ) ){ \
+				if( bli_trsm_my_iter( i, thread ) ){ \
 \
 				ctype* restrict a11; \
 				ctype* restrict a12; \
@@ -440,8 +453,8 @@ void PASTEMAC(ch,varname) \
 				/* Save addresses of next panels of A and B to the auxinfo_t
 				   object. NOTE: We swap the values for A and B since the
 				   triangular "A" matrix is actually contained within B. */ \
-				bli_auxinfo_set_next_a( b2, aux ); \
-				bli_auxinfo_set_next_b( a2, aux ); \
+				bli_auxinfo_set_next_a( b2, &aux ); \
+				bli_auxinfo_set_next_b( a2, &aux ); \
 \
 				/* Handle interior and edge cases separately. */ \
 				if ( m_cur == MR && n_cur == NR ) \
@@ -495,12 +508,12 @@ void PASTEMAC(ch,varname) \
 			   object.
 			   NOTE: We swap the values for A and B since the triangular
 			   "A" matrix is actually contained within B. */ \
-			bli_auxinfo_set_is_a( istep_b, aux ); \
+			bli_auxinfo_set_is_a( istep_b, &aux ); \
 \
 			/* Loop over the m dimension (MR rows at a time). */ \
 			for ( i = 0; i < m_iter; ++i ) \
 			{ \
-				if( trsm_my_iter( i, thread ) ){ \
+				if( bli_trsm_my_iter( i, thread ) ){ \
 \
 				ctype* restrict a2; \
 \
@@ -520,8 +533,8 @@ void PASTEMAC(ch,varname) \
 				/* Save addresses of next panels of A and B to the auxinfo_t
 				   object. NOTE: We swap the values for A and B since the
 				   triangular "A" matrix is actually contained within B. */ \
-				bli_auxinfo_set_next_a( b2, aux ); \
-				bli_auxinfo_set_next_b( a2, aux ); \
+				bli_auxinfo_set_next_a( b2, &aux ); \
+				bli_auxinfo_set_next_b( a2, &aux ); \
 \
 				/* Handle interior and edge cases separately. */ \
 				if ( m_cur == MR && n_cur == NR ) \
