@@ -1,11 +1,12 @@
 /*
 
-   BLIS    
+   BLIS
    An object-based framework for developing high-performance BLAS-like
    libraries.
 
    Copyright (C) 2014, The University of Texas at Austin
-   Copyright (C) 2016 Hewlett Packard Enterprise Development LP
+   Copyright (C) 2016, Hewlett Packard Enterprise Development LP
+   Copyright (C) 2018, Advanced Micro Devices, Inc.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are
@@ -77,6 +78,18 @@ typedef unsigned long int guint_t;
 
 typedef  gint_t  bool_t;
 
+
+// -- Boolean values --
+
+#ifndef TRUE
+  #define TRUE  1
+#endif
+
+#ifndef FALSE
+  #define FALSE 0
+#endif
+
+
 // -- Special-purpose integers --
 
 // This cpp guard provides a temporary hack to allow libflame
@@ -159,9 +172,9 @@ typedef dcomplex atom_t;
 // because they also occur in bli_slamch() and bli_dlamch().
 
 // Define f77_int depending on what size of integer was requested.
-#if   BLIS_BLAS2BLIS_INT_TYPE_SIZE == 32
+#if   BLIS_BLAS_INT_TYPE_SIZE == 32
 typedef int32_t   f77_int;
-#elif BLIS_BLAS2BLIS_INT_TYPE_SIZE == 64
+#elif BLIS_BLAS_INT_TYPE_SIZE == 64
 typedef int64_t   f77_int;
 #else
 typedef long int  f77_int;
@@ -198,11 +211,11 @@ typedef dcomplex  f77_dcomplex;
   12 ~ 10  Target numerical datatype
            - 10: domain    (0 == real, 1 == complex)
            - 11: precision (0 == single, 1 == double)
-           - 12: unused
+           - 12: used to encode integer, constant types
   15 ~ 13  Execution numerical datatype
            - 13: domain    (0 == real, 1 == complex)
            - 14: precision (0 == single, 1 == double)
-           - 15: unused
+           - 15: used to encode integer, constant types
   22 ~ 16  Packed type/status
            - 0 0000 00: not packed
            - 1 0000 00: packed (unspecified; by rows, columns, or vector)
@@ -224,6 +237,10 @@ typedef dcomplex  f77_dcomplex;
            - 1 0110 11: packed imag-only column panels
            - 1 0111 10: packed real+imag row panels
            - 1 0111 11: packed real+imag column panels
+           - 1 1000 10: packed by 1m expanded row panels
+           - 1 1000 11: packed by 1m expanded column panels
+           - 1 1001 10: packed by 1m reordered row panels
+           - 1 1001 11: packed by 1m reordered column panels
        23  Packed panel order if upper-stored
            - 0 == forward order if upper
            - 1 == reverse order if upper
@@ -255,7 +272,11 @@ typedef dcomplex  f77_dcomplex;
 #define BLIS_UNIT_DIAG_SHIFT               8
 #define BLIS_INVERT_DIAG_SHIFT             9
 #define BLIS_TARGET_DT_SHIFT               10
-#define BLIS_EXECUTION_DT_SHIFT            13
+#define   BLIS_TARGET_DOMAIN_SHIFT         10
+#define   BLIS_TARGET_PREC_SHIFT           11
+#define BLIS_EXEC_DT_SHIFT                 13
+#define   BLIS_EXEC_DOMAIN_SHIFT           13
+#define   BLIS_EXEC_PREC_SHIFT             14
 #define BLIS_PACK_SCHEMA_SHIFT             16
 #define   BLIS_PACK_RC_SHIFT               16
 #define   BLIS_PACK_PANEL_SHIFT            17
@@ -283,7 +304,11 @@ typedef dcomplex  f77_dcomplex;
 #define BLIS_UNIT_DIAG_BIT                 ( 0x1  << BLIS_UNIT_DIAG_SHIFT )
 #define BLIS_INVERT_DIAG_BIT               ( 0x1  << BLIS_INVERT_DIAG_SHIFT )
 #define BLIS_TARGET_DT_BITS                ( 0x7  << BLIS_TARGET_DT_SHIFT )
-#define BLIS_EXECUTION_DT_BITS             ( 0x7  << BLIS_EXECUTION_DT_SHIFT )
+#define   BLIS_TARGET_DOMAIN_BIT           ( 0x1  << BLIS_TARGET_DOMAIN_SHIFT )
+#define   BLIS_TARGET_PREC_BIT             ( 0x1  << BLIS_TARGET_PREC_SHIFT )
+#define BLIS_EXEC_DT_BITS                  ( 0x7  << BLIS_EXEC_DT_SHIFT )
+#define   BLIS_EXEC_DOMAIN_BIT             ( 0x1  << BLIS_EXEC_DOMAIN_SHIFT )
+#define   BLIS_EXEC_PREC_BIT               ( 0x1  << BLIS_EXEC_PREC_SHIFT )
 #define BLIS_PACK_SCHEMA_BITS              ( 0x7F << BLIS_PACK_SCHEMA_SHIFT )
 #define   BLIS_PACK_RC_BIT                 ( 0x1  << BLIS_PACK_RC_SHIFT )
 #define   BLIS_PACK_PANEL_BIT              ( 0x1  << BLIS_PACK_PANEL_SHIFT )
@@ -329,6 +354,8 @@ typedef dcomplex  f77_dcomplex;
 #define   BLIS_BITVAL_RO                    ( 0x5  << BLIS_PACK_FORMAT_SHIFT )
 #define   BLIS_BITVAL_IO                    ( 0x6  << BLIS_PACK_FORMAT_SHIFT )
 #define   BLIS_BITVAL_RPI                   ( 0x7  << BLIS_PACK_FORMAT_SHIFT )
+#define   BLIS_BITVAL_1E                    ( 0x8  << BLIS_PACK_FORMAT_SHIFT )
+#define   BLIS_BITVAL_1R                    ( 0x9  << BLIS_PACK_FORMAT_SHIFT )
 #define   BLIS_BITVAL_PACKED_UNSPEC         ( BLIS_PACK_BIT                                                            )
 #define   BLIS_BITVAL_PACKED_ROWS           ( BLIS_PACK_BIT                                                            )
 #define   BLIS_BITVAL_PACKED_COLUMNS        ( BLIS_PACK_BIT                                         | BLIS_PACK_RC_BIT )
@@ -348,6 +375,10 @@ typedef dcomplex  f77_dcomplex;
 #define   BLIS_BITVAL_PACKED_COL_PANELS_IO  ( BLIS_PACK_BIT | BLIS_BITVAL_IO  | BLIS_PACK_PANEL_BIT | BLIS_PACK_RC_BIT )
 #define   BLIS_BITVAL_PACKED_ROW_PANELS_RPI ( BLIS_PACK_BIT | BLIS_BITVAL_RPI | BLIS_PACK_PANEL_BIT                    )
 #define   BLIS_BITVAL_PACKED_COL_PANELS_RPI ( BLIS_PACK_BIT | BLIS_BITVAL_RPI | BLIS_PACK_PANEL_BIT | BLIS_PACK_RC_BIT )
+#define   BLIS_BITVAL_PACKED_ROW_PANELS_1E  ( BLIS_PACK_BIT | BLIS_BITVAL_1E  | BLIS_PACK_PANEL_BIT                    )
+#define   BLIS_BITVAL_PACKED_COL_PANELS_1E  ( BLIS_PACK_BIT | BLIS_BITVAL_1E  | BLIS_PACK_PANEL_BIT | BLIS_PACK_RC_BIT )
+#define   BLIS_BITVAL_PACKED_ROW_PANELS_1R  ( BLIS_PACK_BIT | BLIS_BITVAL_1R  | BLIS_PACK_PANEL_BIT                    )
+#define   BLIS_BITVAL_PACKED_COL_PANELS_1R  ( BLIS_PACK_BIT | BLIS_BITVAL_1R  | BLIS_PACK_PANEL_BIT | BLIS_PACK_RC_BIT )
 #define BLIS_BITVAL_PACK_FWD_IF_UPPER         0x0
 #define BLIS_BITVAL_PACK_REV_IF_UPPER         BLIS_PACK_REV_IF_UPPER_BIT
 #define BLIS_BITVAL_PACK_FWD_IF_LOWER         0x0
@@ -428,7 +459,7 @@ typedef enum
 	BLIS_INT               = BLIS_BITVAL_INT_TYPE,
 	BLIS_CONSTANT          = BLIS_BITVAL_CONST_TYPE,
 	BLIS_DT_LO             = BLIS_FLOAT,
-	BLIS_DT_HI             = BLIS_DCOMPLEX,
+	BLIS_DT_HI             = BLIS_DCOMPLEX
 } num_t;
 
 typedef enum
@@ -469,13 +500,17 @@ typedef enum
 	BLIS_PACKED_COL_PANELS_IO  = BLIS_BITVAL_PACKED_COL_PANELS_IO,
 	BLIS_PACKED_ROW_PANELS_RPI = BLIS_BITVAL_PACKED_ROW_PANELS_RPI,
 	BLIS_PACKED_COL_PANELS_RPI = BLIS_BITVAL_PACKED_COL_PANELS_RPI,
+	BLIS_PACKED_ROW_PANELS_1E  = BLIS_BITVAL_PACKED_ROW_PANELS_1E,
+	BLIS_PACKED_COL_PANELS_1E  = BLIS_BITVAL_PACKED_COL_PANELS_1E,
+	BLIS_PACKED_ROW_PANELS_1R  = BLIS_BITVAL_PACKED_ROW_PANELS_1R,
+	BLIS_PACKED_COL_PANELS_1R  = BLIS_BITVAL_PACKED_COL_PANELS_1R
 } pack_t;
 
 // We combine row and column packing into one "type", and we start
 // with BLIS_PACKED_ROW_PANELS, _COLUMN_PANELS. We also count the
 // schema pair for "4ms" (4m separated), because its bit value has
 // been reserved, even though we don't use it.
-#define BLIS_NUM_PACK_SCHEMA_TYPES 8
+#define BLIS_NUM_PACK_SCHEMA_TYPES 10
 
 
 // -- Pack order type --
@@ -497,237 +532,18 @@ typedef enum
 	BLIS_BUFFER_FOR_A_BLOCK = BLIS_BITVAL_BUFFER_FOR_A_BLOCK,
 	BLIS_BUFFER_FOR_B_PANEL = BLIS_BITVAL_BUFFER_FOR_B_PANEL,
 	BLIS_BUFFER_FOR_C_PANEL = BLIS_BITVAL_BUFFER_FOR_C_PANEL,
-	BLIS_BUFFER_FOR_GEN_USE = BLIS_BITVAL_BUFFER_FOR_GEN_USE,
+	BLIS_BUFFER_FOR_GEN_USE = BLIS_BITVAL_BUFFER_FOR_GEN_USE
 } packbuf_t;
 
 
-//
-// -- BLIS misc. structure types -----------------------------------------------
-//
+// -- Partitioning direction --
 
-// -- Mutex type --
-
-typedef struct mtx_s mtx_t;
-
-// -- Pool block type --
-
-typedef struct
+typedef enum
 {
-	void* buf_sys;
-	void* buf_align;
-} pblk_t;
+	BLIS_FWD,
+	BLIS_BWD
+} dir_t;
 
-// -- Pool type --
-
-typedef struct
-{
-	pblk_t* block_ptrs;
-	dim_t   block_ptrs_len;
-
-	dim_t   top_index;
-	dim_t   num_blocks;
-
-	siz_t   block_size;
-	siz_t   align_size;
-} pool_t;
-
-// -- Memory broker object type --
-
-typedef struct membrk_s membrk_t;
-/*
-{
-	pool_t    pools[3];
-	mtx_t     mutex;
-
-	malloc_ft malloc_fp;
-	free_ft   free_fp;
-} membrk_t;
-*/
-
-// -- Memory object type --
-
-typedef struct mem_s
-{
-	pblk_t    pblk;
-	packbuf_t buf_type;
-	pool_t*   pool;
-	membrk_t* membrk;
-	siz_t     size;
-} mem_t;
-
-// -- Blocksize object type --
-
-typedef struct blksz_s
-{
-	// Primary blocksize values.
-	dim_t  v[BLIS_NUM_FP_TYPES];
-
-	// Blocksize extensions.
-	dim_t  e[BLIS_NUM_FP_TYPES];
-
-} blksz_t;
-
-// -- Function pointer object type --
-
-typedef struct func_s
-{
-	// Kernel function address.
-	void*  ptr[BLIS_NUM_FP_TYPES];
-
-} func_t;
-
-// -- Multi-boolean object type --
-
-typedef struct mbool_s
-{
-	bool_t  v[BLIS_NUM_FP_TYPES];
-
-} mbool_t;
-
-// -- Auxiliary kernel info type --
-
-// Note: This struct is used by macro-kernels to package together extra
-// parameter values that may be of use to the micro-kernel without
-// cluttering up the micro-kernel interface itself.
-
-typedef struct
-{
-	// The pack schemas of A and B.
-	pack_t schema_a;
-	pack_t schema_b;
-
-	// Pointers to the micro-panels of A and B which will be used by the
-	// next call to the micro-kernel.
-	void*  a_next;
-	void*  b_next;
-
-	// The imaginary strides of A and B.
-	inc_t  is_a;
-	inc_t  is_b;
-
-} auxinfo_t;
-
-
-
-//
-// -- BLIS object type definitions ---------------------------------------------
-//
-
-typedef struct obj_s
-{
-	// Basic fields
-	struct obj_s* root;
-
-	dim_t         off[2];
-	dim_t         dim[2];
-	doff_t        diag_off;
-
-	objbits_t     info;
-	siz_t         elem_size;
-
-	void*         buffer;
-	inc_t         rs;
-	inc_t         cs;
-	inc_t         is;
-
-	// Bufferless scalar storage
-	atom_t        scalar;
-
-	// Pack-related fields
-	mem_t         pack_mem; // cached memory region for packing
-	dim_t         m_padded; // m dimension of matrix, including any padding
-	dim_t         n_padded; // n dimension of matrix, including any padding
-	inc_t         ps;       // panel stride (distance to next panel)
-	inc_t         pd;       // panel dimension (the "width" of a panel:
-	                        // usually MR or NR)
-	dim_t         m_panel;  // m dimension of a "full" panel
-	dim_t         n_panel;  // n dimension of a "full" panel
-} obj_t;
-
-
-// Define these macros here since they must be updated if contents of
-// obj_t changes.
-#define bli_obj_init_basic_shallow_copy_of( a, b ) \
-{ \
-	(b).root      = (a).root; \
-\
-	(b).off[0]    = (a).off[0]; \
-	(b).off[1]    = (a).off[1]; \
-	(b).dim[0]    = (a).dim[0]; \
-	(b).dim[1]    = (a).dim[1]; \
-	(b).diag_off  = (a).diag_off; \
-\
-	(b).info      = (a).info; \
-	(b).elem_size = (a).elem_size; \
-\
-	(b).buffer    = (a).buffer; \
-	(b).rs        = (a).rs; \
-	(b).cs        = (a).cs; \
-	(b).is        = (a).is; \
-\
-	(b).scalar    = (a).scalar; \
-\
-	/* We must NOT copy pack_mem field since this macro forms the basis of
-	   bli_obj_alias_to(), which is used in packm_init(). There, we want to
-	   copy the basic fields of the obj_t but PRESERVE the pack_mem field
-	   of the destination object since it holds the "cached" mem_t object
-	   and buffer. The other fields, such as padded dimensions, are always
-	   set by bli_packm_init(), so we don't need to copy them either. */ \
-}
-
-#define bli_obj_init_full_shallow_copy_of( a, b ) \
-{ \
-	/* This macro implements a full alias (shallow copy) that copies all
-	   fields of the obj_t struct. */ \
-	bli_obj_init_basic_shallow_copy_of( a, b ); \
-\
-	(b).pack_mem  = (a).pack_mem; \
-	(b).m_padded  = (a).m_padded; \
-	(b).n_padded  = (a).n_padded; \
-	(b).ps        = (a).ps; \
-	(b).pd        = (a).pd; \
-	(b).m_panel   = (a).m_panel; \
-	(b).n_panel   = (a).n_panel; \
-}
-
-#define bli_obj_init_subpart_from( a, b ) \
-{ \
-	(b).root      = (a).root; \
-\
-	(b).off[0]    = (a).off[0]; \
-	(b).off[1]    = (a).off[1]; \
-	/* Avoid copying m since it will be overwritten. */ \
-	/* Avoid copying n since it will be overwritten. */ \
-	(b).diag_off  = (a).diag_off; \
-\
-	(b).info      = (a).info; \
-	(b).elem_size = (a).elem_size; \
-\
-	(b).buffer    = (a).buffer; \
-	(b).rs        = (a).rs; \
-	(b).cs        = (a).cs; \
-	(b).is        = (a).is; \
-\
-	(b).scalar    = (a).scalar; \
-\
-	/* We want to copy the pack_mem field here because this macro is used
-	   when creating subpartitions, including those of packed objects. In
-	   those situations, we want the subpartition to inherit the pack_mem
-	   field of its parent, as well as other related fields such as the
-	   padded dimensions. */ \
-	(b).pack_mem  = (a).pack_mem; \
-	(b).m_padded  = (a).m_padded; \
-	(b).n_padded  = (a).n_padded; \
-	(b).pd        = (a).pd; \
-	(b).ps        = (a).ps; \
-	(b).m_panel   = (a).m_panel; \
-	(b).n_panel   = (a).n_panel; \
-}
-
-
-//
-// -- Other BLIS enumerated type definitions -----------------------------------
-//
 
 // -- Subpartition type --
 
@@ -782,27 +598,39 @@ typedef enum
 #define BLIS_MACH_PARAM_FIRST  BLIS_MACH_EPS
 #define BLIS_MACH_PARAM_LAST   BLIS_MACH_EPS2
 
+
 // -- Induced method types --
 
 typedef enum
 {
 	BLIS_3MH = 0,
-	BLIS_3M3,
-	BLIS_3M2,
 	BLIS_3M1,
 	BLIS_4MH,
 	BLIS_4M1B,
 	BLIS_4M1A,
-	BLIS_NAT,
+	BLIS_1M,
+	BLIS_NAT
 } ind_t;
 
 #define BLIS_NUM_IND_METHODS (BLIS_NAT+1)
 
+// These are used in bli_*_oapi.c to construct the ind_t values from
+// the induced method substrings that go into function names.
+#define bli_3mh  BLIS_3MH
+#define bli_3m1  BLIS_3M1
+#define bli_4mh  BLIS_4MH
+#define bli_4mb  BLIS_4M1B
+#define bli_4m1  BLIS_4M1A
+#define bli_1m   BLIS_1M
+#define bli_nat  BLIS_NAT
+
+
 // -- Kernel ID types --
 
 typedef enum
 {
 	BLIS_ADDV_KER  = 0,
+	BLIS_AMAXV_KER,
 	BLIS_AXPBYV_KER,
 	BLIS_AXPYV_KER,
 	BLIS_COPYV_KER,
@@ -814,10 +642,11 @@ typedef enum
 	BLIS_SETV_KER,
 	BLIS_SUBV_KER,
 	BLIS_SWAPV_KER,
-	BLIS_XPBYV_KER,
+	BLIS_XPBYV_KER
 } l1vkr_t;
 
-#define BLIS_NUM_LEVEL1V_KERS 13
+#define BLIS_NUM_LEVEL1V_KERS 14
+
 
 typedef enum
 {
@@ -825,33 +654,124 @@ typedef enum
 	BLIS_DOTAXPYV_KER,
 	BLIS_AXPYF_KER,
 	BLIS_DOTXF_KER,
-	BLIS_DOTXAXPYF_KER,
+	BLIS_DOTXAXPYF_KER
 } l1fkr_t;
 
 #define BLIS_NUM_LEVEL1F_KERS 5
 
+
+typedef enum
+{
+	BLIS_PACKM_0XK_KER  = 0,
+	BLIS_PACKM_1XK_KER  = 1,
+	BLIS_PACKM_2XK_KER  = 2,
+	BLIS_PACKM_3XK_KER  = 3,
+	BLIS_PACKM_4XK_KER  = 4,
+	BLIS_PACKM_5XK_KER  = 5,
+	BLIS_PACKM_6XK_KER  = 6,
+	BLIS_PACKM_7XK_KER  = 7,
+	BLIS_PACKM_8XK_KER  = 8,
+	BLIS_PACKM_9XK_KER  = 9,
+	BLIS_PACKM_10XK_KER = 10,
+	BLIS_PACKM_11XK_KER = 11,
+	BLIS_PACKM_12XK_KER = 12,
+	BLIS_PACKM_13XK_KER = 13,
+	BLIS_PACKM_14XK_KER = 14,
+	BLIS_PACKM_15XK_KER = 15,
+	BLIS_PACKM_16XK_KER = 16,
+	BLIS_PACKM_17XK_KER = 17,
+	BLIS_PACKM_18XK_KER = 18,
+	BLIS_PACKM_19XK_KER = 19,
+	BLIS_PACKM_20XK_KER = 20,
+	BLIS_PACKM_21XK_KER = 21,
+	BLIS_PACKM_22XK_KER = 22,
+	BLIS_PACKM_23XK_KER = 23,
+	BLIS_PACKM_24XK_KER = 24,
+	BLIS_PACKM_25XK_KER = 25,
+	BLIS_PACKM_26XK_KER = 26,
+	BLIS_PACKM_27XK_KER = 27,
+	BLIS_PACKM_28XK_KER = 28,
+	BLIS_PACKM_29XK_KER = 29,
+	BLIS_PACKM_30XK_KER = 30,
+	BLIS_PACKM_31XK_KER = 31,
+
+	BLIS_UNPACKM_0XK_KER  = 0,
+	BLIS_UNPACKM_1XK_KER  = 1,
+	BLIS_UNPACKM_2XK_KER  = 2,
+	BLIS_UNPACKM_3XK_KER  = 3,
+	BLIS_UNPACKM_4XK_KER  = 4,
+	BLIS_UNPACKM_5XK_KER  = 5,
+	BLIS_UNPACKM_6XK_KER  = 6,
+	BLIS_UNPACKM_7XK_KER  = 7,
+	BLIS_UNPACKM_8XK_KER  = 8,
+	BLIS_UNPACKM_9XK_KER  = 9,
+	BLIS_UNPACKM_10XK_KER = 10,
+	BLIS_UNPACKM_11XK_KER = 11,
+	BLIS_UNPACKM_12XK_KER = 12,
+	BLIS_UNPACKM_13XK_KER = 13,
+	BLIS_UNPACKM_14XK_KER = 14,
+	BLIS_UNPACKM_15XK_KER = 15,
+	BLIS_UNPACKM_16XK_KER = 16,
+	BLIS_UNPACKM_17XK_KER = 17,
+	BLIS_UNPACKM_18XK_KER = 18,
+	BLIS_UNPACKM_19XK_KER = 19,
+	BLIS_UNPACKM_20XK_KER = 20,
+	BLIS_UNPACKM_21XK_KER = 21,
+	BLIS_UNPACKM_22XK_KER = 22,
+	BLIS_UNPACKM_23XK_KER = 23,
+	BLIS_UNPACKM_24XK_KER = 24,
+	BLIS_UNPACKM_25XK_KER = 25,
+	BLIS_UNPACKM_26XK_KER = 26,
+	BLIS_UNPACKM_27XK_KER = 27,
+	BLIS_UNPACKM_28XK_KER = 28,
+	BLIS_UNPACKM_29XK_KER = 29,
+	BLIS_UNPACKM_30XK_KER = 30,
+	BLIS_UNPACKM_31XK_KER = 31
+
+} l1mkr_t;
+
+#define BLIS_NUM_PACKM_KERS   32
+#define BLIS_NUM_UNPACKM_KERS 32
+
+
 typedef enum
 {
 	BLIS_GEMM_UKR = 0,
 	BLIS_GEMMTRSM_L_UKR,
 	BLIS_GEMMTRSM_U_UKR,
 	BLIS_TRSM_L_UKR,
-	BLIS_TRSM_U_UKR,
+	BLIS_TRSM_U_UKR
 } l3ukr_t;
 
 #define BLIS_NUM_LEVEL3_UKRS 5
 
+
 typedef enum
 {
 	BLIS_REFERENCE_UKERNEL = 0,
 	BLIS_VIRTUAL_UKERNEL,
 	BLIS_OPTIMIZED_UKERNEL,
-	BLIS_NOTAPPLIC_UKERNEL,
+	BLIS_NOTAPPLIC_UKERNEL
 } kimpl_t;
 
 #define BLIS_NUM_UKR_IMPL_TYPES 4
 
 
+#if 0
+typedef enum
+{
+	BLIS_JC_IDX = 0,
+	BLIS_PC_IDX,
+	BLIS_IC_IDX,
+	BLIS_JR_IDX,
+	BLIS_IR_IDX,
+	BLIS_PR_IDX
+} thridx_t;
+#endif
+
+#define BLIS_NUM_LOOPS 6
+
+
 // -- Operation ID type --
 
 typedef enum
@@ -866,7 +786,7 @@ typedef enum
 //   value that can be subtracted from the opid_t value to map it
 //   to a zero-based range.
 // This is needed because these level-3 opid_t values are used in
-// bli_ind_query.c to index into arrays.
+// bli_l3_ind.c to index into arrays.
 //
 	BLIS_GEMM = 0,
 	BLIS_HEMM,
@@ -879,7 +799,7 @@ typedef enum
 	BLIS_TRMM,
 	BLIS_TRSM,
 
-	BLIS_NOID,
+	BLIS_NOID
 } opid_t;
 
 #define BLIS_NUM_LEVEL3_OPS 10
@@ -889,6 +809,10 @@ typedef enum
 
 typedef enum
 {
+	// NOTE: the level-3 blocksizes MUST be indexed starting at zero.
+	// At one point, we made this assumption in bli_cntx_set_blkszs()
+	// and friends.
+
 	BLIS_KR = 0,
 	BLIS_MR,
 	BLIS_NR,
@@ -897,14 +821,305 @@ typedef enum
 	BLIS_NC,
 	BLIS_M2, // level-2 blocksize in m dimension
 	BLIS_N2, // level-2 blocksize in n dimension
-	BLIS_1F, // level-1f global fusing factor
 	BLIS_AF, // level-1f axpyf fusing factor
 	BLIS_DF, // level-1f dotxf fusing factor
 	BLIS_XF, // level-1f dotxaxpyf fusing factor
-	BLIS_VF, // level-1v vector fusing factor
+
+	BLIS_NO_PART  // used as a placeholder when blocksizes are not applicable.
 } bszid_t;
 
-#define BLIS_NUM_BLKSZS 13
+#define BLIS_NUM_BLKSZS 11
+
+
+// -- Architecture ID type --
+
+// NOTE: This typedef enum must be kept up-to-date with the arch_t
+// string array in bli_arch.c. Whenever values are added/inserted
+// OR if values are rearranged, be sure to update the string array
+// in bli_arch.c.
+
+typedef enum
+{
+	// Intel
+	BLIS_ARCH_SKX = 0,
+	BLIS_ARCH_KNL,
+	BLIS_ARCH_KNC,
+	BLIS_ARCH_HASWELL,
+	BLIS_ARCH_SANDYBRIDGE,
+	BLIS_ARCH_PENRYN,
+
+	// AMD
+	BLIS_ARCH_ZEN,
+	BLIS_ARCH_EXCAVATOR,
+	BLIS_ARCH_STEAMROLLER,
+	BLIS_ARCH_PILEDRIVER,
+	BLIS_ARCH_BULLDOZER,
+
+	// ARM
+	BLIS_ARCH_CORTEXA57,
+	BLIS_ARCH_CORTEXA53,
+	BLIS_ARCH_CORTEXA15,
+	BLIS_ARCH_CORTEXA9,
+
+	// IBM/Power
+	BLIS_ARCH_POWER7,
+	BLIS_ARCH_BGQ,
+
+	// Generic architecture/configuration
+	BLIS_ARCH_GENERIC
+
+} arch_t;
+
+#define BLIS_NUM_ARCHS 18
+
+
+//
+// -- BLIS misc. structure types -----------------------------------------------
+//
+
+// -- Pool block type --
+
+typedef struct
+{
+	void* buf_sys;
+	void* buf_align;
+} pblk_t;
+
+
+// -- Pool type --
+
+typedef struct
+{
+	pblk_t* block_ptrs;
+	dim_t   block_ptrs_len;
+
+	dim_t   top_index;
+	dim_t   num_blocks;
+
+	siz_t   block_size;
+	siz_t   align_size;
+} pool_t;
+
+
+// -- Memory broker object type --
+
+#include "bli_pthread_wrap.h"
+#include "bli_malloc.h"
+
+typedef struct membrk_s
+{
+	pool_t          pools[3];
+	pthread_mutex_t mutex;
+
+	malloc_ft       malloc_fp;
+	free_ft         free_fp;
+} membrk_t;
+
+
+// -- Memory object type --
+
+typedef struct mem_s
+{
+	pblk_t    pblk;
+	packbuf_t buf_type;
+	pool_t*   pool;
+	membrk_t* membrk;
+	siz_t     size;
+} mem_t;
+
+
+// -- Control tree node type --
+
+struct cntl_s
+{
+	// Basic fields (usually required).
+	opid_t         family;
+	bszid_t        bszid;
+	void*          var_func;
+	struct cntl_s* sub_node;
+
+	// Optional fields (needed only by some operations such as packm).
+	// NOTE: first field of params must be a uint64_t containing the size
+	// of the struct.
+	void*          params;
+
+	// Internal fields that track "cached" data.
+	mem_t          pack_mem;
+};
+typedef struct cntl_s cntl_t;
+
+
+// -- Blocksize object type --
+
+typedef struct blksz_s
+{
+	// Primary blocksize values.
+	dim_t  v[BLIS_NUM_FP_TYPES];
+
+	// Blocksize extensions.
+	dim_t  e[BLIS_NUM_FP_TYPES];
+
+} blksz_t;
+
+
+// -- Function pointer object type --
+
+typedef struct func_s
+{
+	// Kernel function address.
+	void*  ptr[BLIS_NUM_FP_TYPES];
+
+} func_t;
+
+
+// -- Multi-boolean object type --
+
+typedef struct mbool_s
+{
+	bool_t  v[BLIS_NUM_FP_TYPES];
+
+} mbool_t;
+
+
+// -- Auxiliary kernel info type --
+
+// Note: This struct is used by macro-kernels to package together extra
+// parameter values that may be of use to the micro-kernel without
+// cluttering up the micro-kernel interface itself.
+
+typedef struct
+{
+	// The pack schemas of A and B.
+	pack_t schema_a;
+	pack_t schema_b;
+
+	// Pointers to the micro-panels of A and B which will be used by the
+	// next call to the micro-kernel.
+	void*  a_next;
+	void*  b_next;
+
+	// The imaginary strides of A and B.
+	inc_t  is_a;
+	inc_t  is_b;
+
+	// The type to convert to on output.
+	num_t  dt_on_output;
+
+} auxinfo_t;
+
+
+// -- Global scalar constant data struct --
+
+// Note: This struct is used only when statically initializing the
+// global scalar constants in bli_const.c.
+typedef struct constdata_s
+{
+	float    s;
+	double   d;
+	scomplex c;
+	dcomplex z;
+	gint_t   i;
+
+} constdata_t;
+
+
+//
+// -- BLIS object type definitions ---------------------------------------------
+//
+
+typedef struct obj_s
+{
+	// Basic fields
+	struct obj_s* root;
+
+	dim_t         off[2];
+	dim_t         dim[2];
+	doff_t        diag_off;
+
+	objbits_t     info;
+	siz_t         elem_size;
+
+	void*         buffer;
+	inc_t         rs;
+	inc_t         cs;
+	inc_t         is;
+
+	// Bufferless scalar storage
+	atom_t        scalar;
+
+	// Pack-related fields
+	dim_t         m_padded; // m dimension of matrix, including any padding
+	dim_t         n_padded; // n dimension of matrix, including any padding
+	inc_t         ps;       // panel stride (distance to next panel)
+	inc_t         pd;       // panel dimension (the "width" of a panel:
+	                        // usually MR or NR)
+	dim_t         m_panel;  // m dimension of a "full" panel
+	dim_t         n_panel;  // n dimension of a "full" panel
+} obj_t;
+
+// Define these macros here since they must be updated if contents of
+// obj_t changes.
+
+static void bli_obj_init_full_shallow_copy_of( obj_t* a, obj_t* b )
+{
+	b->root      = a->root;
+
+	b->off[0]    = a->off[0];
+	b->off[1]    = a->off[1];
+	b->dim[0]    = a->dim[0];
+	b->dim[1]    = a->dim[1];
+	b->diag_off  = a->diag_off;
+
+	b->info      = a->info;
+	b->elem_size = a->elem_size;
+
+	b->buffer    = a->buffer;
+	b->rs        = a->rs;
+	b->cs        = a->cs;
+	b->is        = a->is;
+
+	b->scalar    = a->scalar;
+
+	//b->pack_mem  = a->pack_mem;
+	b->m_padded  = a->m_padded;
+	b->n_padded  = a->n_padded;
+	b->ps        = a->ps;
+	b->pd        = a->pd;
+	b->m_panel   = a->m_panel;
+	b->n_panel   = a->n_panel;
+}
+
+static void bli_obj_init_subpart_from( obj_t* a, obj_t* b )
+{
+	b->root      = a->root;
+
+	b->off[0]    = a->off[0];
+	b->off[1]    = a->off[1];
+	// Avoid copying m and n since they will be overwritten.
+	//b->dim[0]    = a->dim[0];
+	//b->dim[1]    = a->dim[1];
+	b->diag_off  = a->diag_off;
+
+	b->info      = a->info;
+	b->elem_size = a->elem_size;
+
+	b->buffer    = a->buffer;
+	b->rs        = a->rs;
+	b->cs        = a->cs;
+	b->is        = a->is;
+
+	b->scalar    = a->scalar;
+
+	// Avoid copying pack_mem entry.
+	// FGVZ: You should probably make sure this is right.
+	//b->pack_mem  = a->pack_mem;
+	b->m_padded  = a->m_padded;
+	b->n_padded  = a->n_padded;
+	b->ps        = a->ps;
+	b->pd        = a->pd;
+	b->m_panel   = a->m_panel;
+	b->n_panel   = a->n_panel;
+}
 
 
 // -- Context type --
@@ -921,23 +1136,34 @@ typedef struct cntx_s
 	func_t    l1f_kers[ BLIS_NUM_LEVEL1F_KERS ];
 	func_t    l1v_kers[ BLIS_NUM_LEVEL1V_KERS ];
 
-	func_t    packm_ukrs;
+	func_t    packm_kers[ BLIS_NUM_PACKM_KERS ];
+	func_t    unpackm_kers[ BLIS_NUM_UNPACKM_KERS ];
 
 	ind_t     method;
-	pack_t    schema_a;
-	pack_t    schema_b;
-	pack_t    schema_c;
+	pack_t    schema_a_block;
+	pack_t    schema_b_panel;
+	pack_t    schema_c_panel;
 
 	membrk_t* membrk;
 } cntx_t;
 
 
+// -- Runtime type --
+
+typedef struct rntm_s
+{
+	dim_t     num_threads;
+	dim_t     thrloop[ BLIS_NUM_LOOPS ];
+
+} rntm_t;
+
+
 // -- Error types --
 
 typedef enum
 {
 	BLIS_NO_ERROR_CHECKING = 0,
-	BLIS_FULL_ERROR_CHECKING,
+	BLIS_FULL_ERROR_CHECKING
 } errlev_t;
 
 typedef enum
@@ -973,6 +1199,7 @@ typedef enum
 	BLIS_INCONSISTENT_DATATYPES                = ( -36),
 	BLIS_EXPECTED_REAL_PROJ_OF                 = ( -37),
 	BLIS_EXPECTED_REAL_VALUED_OBJECT           = ( -38),
+	BLIS_INCONSISTENT_PRECISIONS               = ( -39),
 
 	// Dimension-specific errors
 	BLIS_NONCONFORMAL_DIMENSIONS               = ( -40),
@@ -1016,7 +1243,6 @@ typedef enum
 
 	// Memory allocator errors
 	BLIS_INVALID_PACKBUF                       = (-120),
-	BLIS_REQUESTED_CONTIG_BLOCK_TOO_BIG        = (-121),
 	BLIS_EXHAUSTED_CONTIG_MEMORY_POOL          = (-122),
 	BLIS_INSUFFICIENT_STACK_BUF_SIZE           = (-123),
 	BLIS_ALIGNMENT_NOT_POWER_OF_TWO            = (-124),
@@ -1025,7 +1251,18 @@ typedef enum
 	// Object-related errors
 	BLIS_EXPECTED_OBJECT_ALIAS                 = (-130),
 
-	BLIS_ERROR_CODE_MAX                        = (-140)
+	// Architecture-related errors
+	BLIS_INVALID_ARCH_ID                       = (-140),
+
+	// Blocksize-related errors
+	BLIS_MC_DEF_NONMULTIPLE_OF_MR              = (-150),
+	BLIS_MC_MAX_NONMULTIPLE_OF_MR              = (-151),
+	BLIS_NC_DEF_NONMULTIPLE_OF_NR              = (-152),
+	BLIS_NC_MAX_NONMULTIPLE_OF_NR              = (-153),
+	BLIS_KC_DEF_NONMULTIPLE_OF_KR              = (-154),
+	BLIS_KC_MAX_NONMULTIPLE_OF_KR              = (-155),
+
+	BLIS_ERROR_CODE_MAX                        = (-160)
 } err_t;
 
 #endif
