@@ -4,11 +4,12 @@
 # https://github.com/pypa/warehouse/blob/master/warehouse/__about__.py
 
 __name__ = 'blis'
-__title__ = 'blis'
-__version__ = '0.0.6'
-__summary__ = 'Cython bindings for Blis linear algebra'
-__uri__ = 'https://github.com/spacy-io/cython-blis'
+__version__ = '0.1.0'
+__summary__ = "Practical Machine Learning for NLP"
+__summary__ = "The Blis BLAS-like linear algebra library, as a self-contained C-extension.",
+__uri__ = 'https://github.com/explosion/cython-blis'
 __author__ = 'Matthew Honnibal'
-__email__ = 'honnibal@gmail.com'
+__email__ = 'matt@explosion.ai'
 __license__ = 'MIT'
+__title__ = "blis"
 __release__ = True
